package com.simuwang.base.pojo.dto.report;

import com.simuwang.base.common.conts.Constants;

import java.io.Serial;
import java.io.Serializable;

/**
 * @author wangzaijun
 * @date 2024/10/14 8:52
 * @description 自定义传递的表格对象
 */
public class CustomExcelTable implements Serializable {
    @Serial
    private static final long serialVersionUID = Constants.DEFAULT_SERIAL_ID;
    /**
     * 表格标识，数据入表的逻辑判断，不能为空
     */
    private final String key;
    /**
     * 表格识别标题（equals或者contains识别），不能为空，多个用逗号分隔
     */
    private final String titles;
    /**
     * 表格识别列数（不能小于等于0）
     */
    private final int colCount;
    /**
     * 表格识别的行数（除了标题行包含表头，如果为0则不限制行数）
     */
    private final int rowCount;
    /**
     * 表格识别起始列，默认第二列开始
     */
    private final int startCol;

    public CustomExcelTable(String key, String titles, int colCount) {
        this(key, titles, colCount, 0);
    }

    public CustomExcelTable(String key, String titles, int colCount, int rowCount) {
        this(key, titles, colCount, rowCount, 1);
    }

    public CustomExcelTable(String key, String titles, int colCount, int rowCount, int startCol) {
        if (key == null) {
            throw new NullPointerException("table key is null.");
        }
        if (titles == null) {
            throw new NullPointerException("table titles is null.");
        }
        if (colCount <= 0) {
            throw new IndexOutOfBoundsException("table col index out of.");
        }
        this.key = key;
        this.titles = titles;
        this.colCount = colCount;
        this.rowCount = rowCount <= 0 ? Integer.MAX_VALUE : rowCount;
        this.startCol = Math.max(startCol, 1);
    }

    public String getKey() {
        return key;
    }

    public String getTitles() {
        return titles;
    }

    public int getColCount() {
        return colCount;
    }

    public int getStartCol() {
        return startCol;
    }

    public int getRowCount() {
        return rowCount;
    }
}
