package com.simuwang.daq.components;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.read.metadata.holder.ReadSheetHolder;
import com.simuwang.base.pojo.dto.report.CustomExcelTable;
import com.simuwang.base.pojo.dto.report.SimpleTable;

import java.io.Closeable;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * @author wangzaijun
 * @date 2024/10/12 9:17
 * @description 自定义的excel多sheet解析事件监听器
 */
public class CustomExcelMultiSheetListener extends AnalysisEventListener<LinkedHashMap<Integer, Object>> implements Closeable {
    /**
     * 所有提取出来的表格
     */
    private final List<SimpleTable> tables = ListUtil.list(false);
    /**
     * 当前表格（每次标题变更都需要重新赋值）
     */
    private SimpleTable table;
    /**
     * 当前提取表格的配置（每次标题变更都需要重新赋值）
     */
    private CustomExcelTable customExcelTable;

    public List<SimpleTable> getTables() {
        return tables;
    }

    @Override
    public void invoke(LinkedHashMap<Integer, Object> row, AnalysisContext analysisContext) {
        ReadSheetHolder sheetHolder = analysisContext.readSheetHolder();
        String sheetName = sheetHolder.getSheetName();
        if (sheetName.contains("封面") && sheetHolder.getSheetNo() == 0) {
            return;
        }
        @SuppressWarnings("unchecked")
        List<CustomExcelTable> customExcelTables = (List<CustomExcelTable>) analysisContext.getCustom();
        // 表格行的第一个位置的字符串
        String title = ReportParseUtils.cleaningValue(row.get(1));
        if (title != null) {
            // 判断是否标题行：如果是标题行或者是另外一个表格的标题行，则重新声明一个表格并追加到解析结果中
            for (CustomExcelTable customExcelTable : customExcelTables) {
                String tableTitles = customExcelTable.getTitles();
                long containCount = StrUtil.split(tableTitles, ",").stream().filter(title::contains).count();
                if (title.equals(tableTitles) || title.contains(tableTitles) || containCount > 0) {
                    this.table = new SimpleTable(customExcelTable.getKey(), title, customExcelTable.getColCount());
                    this.customExcelTable = customExcelTable;
                    this.tables.add(this.table);
                    return;
                }
            }
        }
        // 表格不为空 并且 识别的表格行数比配置的表格行数小，则把当前行追加到表格的行中
        if (this.table != null && this.table.getRowCount() < this.customExcelTable.getRowCount()) {
            List<String> tableRow = ListUtil.list(true);
            int colCount = this.table.getColCount() <= 0 ? row.size() : this.table.getColCount() + this.customExcelTable.getStartCol();
            for (int i = this.customExcelTable.getStartCol(); i < colCount; i++) {
                String cell = ReportParseUtils.cleaningValue(row.get(i), false);
                if (cell != null) {
                    cell = cell.replace(" ", "")
                            .replace("（", "(").replace("）", ")")
                            .replace("：", ":").replace("；", ";");
                }
                tableRow.add(cell);
            }
            this.table.addRow(tableRow);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {

    }

    @Override
    public void close() throws IOException {
        this.tables.clear();
    }
}
