package com.simuwang.manage.api.databoard;

import com.simuwang.base.pojo.dto.query.DataboardQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.daq.service.EmailParseService;
import com.simuwang.logging.SystemLog;
import com.smppw.common.pojo.ResultVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 数据看板
 * Author:   chenjianhua
 * Date:     2024/10/14 10:33
 * Description: ${DESCRIPTION}
 */
@SystemLog("数据看板")
@RestController
@RequestMapping("/v1/data-board")
public class DataboardController {

    @Autowired
    private EmailParseService emailParseService;

    /**
     * 邮件总数
     * @param databoardQuery
     * @return
     */
    @SystemLog("邮件总数")
    @RequestMapping("/email-count")
    public ResultVo<List<NameValueVO>> emailCount(DataboardQuery databoardQuery){
        List<NameValueVO> data = emailParseService.searchEmailCount(databoardQuery);
        return new ResultVo<>(data);
    }

    /**
     * 邮件类型分布
     * @param databoardQuery
     * @return
     */
    @RequestMapping("/email-type-count")
    public ResultVo<List<NameValueVO>> emailTypeCount(DataboardQuery databoardQuery){
        List<NameValueVO> data = emailParseService.searchEmailTypeCount(databoardQuery);
        return new ResultVo<>(data);
    }

    /**
     * 解析错误邮件分析
     * @param databoardQuery
     * @return
     */
    @RequestMapping("/parse-fail-analysis")
    public ResultVo<EmailParseFailAnalysisVO> parseFailAnalysis(DataboardQuery databoardQuery){
        EmailParseFailAnalysisVO data = emailParseService.parseFailAnalysis(databoardQuery);
        return new ResultVo<>(data);
    }

    /**
     * 数据概况
     * @param databoardQuery
     * @return
     */
    @RequestMapping("/data-overview")
    public ResultVo<EmailParseDataViewVO> dataOverview(DataboardQuery databoardQuery){
        EmailParseDataViewVO data = emailParseService.dataOverview(databoardQuery);
        return new ResultVo<>(data);
    }

}
